'use strict';

import os from 'os';
import systeminformation from 'systeminformation';

/**
 * System monitor plugin for statscloud client
 */
export default class Plugin {

  /**
   * Constructs plugin
   */
  constructor(statscloud, settings = {}) {
    this._statscloud = statscloud;
    const interval = (settings.collectIntervalInSeconds || 10) * 1000;
    this._collectInterval = setInterval(this._collectStats.bind(this), interval);
  }

  _bytesToGigabytes(bytes) {
    return bytes / 1024 / 1024 / 1024;
  }

  _bytesToMegabytes(bytes) {
    return bytes / 1024 / 1024;
  }

  /**
   * Collecting stats and sending to statscloud
   */
  async _collectStats() {
    if (!this._statscloud.meter) {
      return;
    }
    const meminfo = await systeminformation.mem();
    this._statscloud.meter.recordEvent('system-monitor.memory.available-gb', this._bytesToGigabytes(meminfo.available));
    this._statscloud.meter.recordEvent('system-monitor.memory.active-gb', this._bytesToGigabytes(meminfo.active));
    this._statscloud.meter.recordEvent('system-monitor.memory.active-percent', 100 * meminfo.active / meminfo.total);
    this._statscloud.meter.recordEvent('system-monitor.memory.swap-gb', this._bytesToGigabytes(meminfo.swapused));

    const cpuinfo = await systeminformation.currentLoad();
    this._statscloud.meter.recordEvent('system-monitor.cpu.used-percent', cpuinfo.currentload);
    this._statscloud.meter.recordEvent('system-monitor.system.load', cpuinfo.avgload);

    const batteryinfo = await systeminformation.battery();
    if (batteryinfo.hasbattery) {
      this._statscloud.meter.recordEvent('system-monitor.battery.percent', batteryinfo.percent);
    }

    const diskinfo = await systeminformation.fsSize();
    const recordDiskInfo = (info) => {
      this._statscloud.meter.recordEvent('system-monitor.disk.used-percent', info.use);
      this._statscloud.meter.recordEvent('system-monitor.disk.free-gb',
        this._bytesToGigabytes(info.size - info.used));
      this._statscloud.meter.recordEvent('system-monitor.disk.free-percent', 100 - info.use);
    };
    const rootInfo = diskinfo.find((info) => info.mount === '/');
    if (rootInfo) {
      recordDiskInfo(rootInfo);
    } else if (diskinfo.length) {
      recordDiskInfo(diskinfo[0]);
    }

    let rx = 0;
    let tx = 0;
    const interfaces = await systeminformation.networkInterfaces();
    await Promise.all(interfaces.map(async (i) => {
      const networkinfo = await systeminformation.networkStats(i.iface);
      if (networkinfo.rx_sec > 0) {
        rx += networkinfo.rx_sec;
      }
      if (networkinfo.tx_sec > 0) {
        tx += networkinfo.tx_sec;
      }
    }));
    if (rx > 0) {
      this._statscloud.meter.recordEvent('system-monitor.net.rx-mb-per-s', this._bytesToMegabytes(rx));
    }
    if (tx > 0) {
      this._statscloud.meter.recordEvent('system-monitor.net.tx-mb-per-s', this._bytesToMegabytes(tx));
    }
  }
}
